@@ -46,7 +46,7 @@ def get_named_object(module_name, member_name=None):
     if member_name:
         # Give __import__ a from_list.  It will return the last module in
         # the dotted module name.
-        attr_chain = member_name.split('.')
+        attr_chain = member_name.split(".")
         from_list = attr_chain[:1]
         obj = __import__(module_name, {}, {}, from_list)
         for attr in attr_chain:
@@ -68,7 +68,7 @@ def iter_suite_tests(suite):
         for item in suite:
             yield from iter_suite_tests(item)
     else:
-        raise Exception(f'unknown type {type(suite)!r} for object {suite!r}')
+        raise Exception(f"unknown type {type(suite)!r} for object {suite!r}")
 
 
 def clone_test(test, new_id):
@@ -164,8 +164,9 @@ def multiply_tests(tests, scenarios, result):
     return result
 
 
-def permute_tests_for_extension(standard_tests, loader, py_module_name,
-                                ext_module_name):
+def permute_tests_for_extension(
+    standard_tests, loader, py_module_name, ext_module_name
+):
     """Helper for permutating tests against an extension module.
 
     This is meant to be used inside a modules 'load_tests()' function. It will
@@ -188,7 +189,7 @@ def permute_tests_for_extension(standard_tests, loader, py_module_name,
     """
     py_module = get_named_object(py_module_name)
     scenarios = [
-        ('python', {'module': py_module}),
+        ("python", {"module": py_module}),
     ]
     suite = loader.suiteClass()
     try:
@@ -196,15 +197,18 @@ def permute_tests_for_extension(standard_tests, loader, py_module_name,
     except ModuleNotFoundError:
         pass
     else:
-        scenarios.append(('C', {'module': get_named_object(ext_module_name)}))
+        scenarios.append(("C", {"module": get_named_object(ext_module_name)}))
     result = multiply_tests(standard_tests, scenarios, suite)
     return result
 
 
 def load_tests(loader, standard_tests, pattern):
     return permute_tests_for_extension(
-        standard_tests, loader, 'fastbencode._bencode_py',
-        'fastbencode._bencode_pyx')
+        standard_tests,
+        loader,
+        "fastbencode._bencode_py",
+        "fastbencode._bencode_pyx",
+    )
 
 
 class RecursionLimit:
@@ -223,7 +227,6 @@ class RecursionLimit:
 
 
 class TestBencodeDecode(TestCase):
-
     module = None
 
     def _check(self, expected, source):
@@ -234,61 +237,63 @@ class TestBencodeDecode(TestCase):
         self.assertRaises(exc, self.module.bdecode, bad)
 
     def test_int(self):
-        self._check(0, b'i0e')
-        self._check(4, b'i4e')
-        self._check(123456789, b'i123456789e')
-        self._check(-10, b'i-10e')
-        self._check(int('1' * 1000), b'i' + (b'1' * 1000) + b'e')
+        self._check(0, b"i0e")
+        self._check(4, b"i4e")
+        self._check(123456789, b"i123456789e")
+        self._check(-10, b"i-10e")
+        self._check(int("1" * 1000), b"i" + (b"1" * 1000) + b"e")
 
     def test_long(self):
-        self._check(12345678901234567890, b'i12345678901234567890e')
-        self._check(-12345678901234567890, b'i-12345678901234567890e')
+        self._check(12345678901234567890, b"i12345678901234567890e")
+        self._check(-12345678901234567890, b"i-12345678901234567890e")
 
     def test_malformed_int(self):
-        self._run_check_error(ValueError, b'ie')
-        self._run_check_error(ValueError, b'i-e')
-        self._run_check_error(ValueError, b'i-010e')
-        self._run_check_error(ValueError, b'i-0e')
-        self._run_check_error(ValueError, b'i00e')
-        self._run_check_error(ValueError, b'i01e')
-        self._run_check_error(ValueError, b'i-03e')
-        self._run_check_error(ValueError, b'i')
-        self._run_check_error(ValueError, b'i123')
-        self._run_check_error(ValueError, b'i341foo382e')
+        self._run_check_error(ValueError, b"ie")
+        self._run_check_error(ValueError, b"i-e")
+        self._run_check_error(ValueError, b"i-010e")
+        self._run_check_error(ValueError, b"i-0e")
+        self._run_check_error(ValueError, b"i00e")
+        self._run_check_error(ValueError, b"i01e")
+        self._run_check_error(ValueError, b"i-03e")
+        self._run_check_error(ValueError, b"i")
+        self._run_check_error(ValueError, b"i123")
+        self._run_check_error(ValueError, b"i341foo382e")
 
     def test_string(self):
-        self._check(b'', b'0:')
-        self._check(b'abc', b'3:abc')
-        self._check(b'1234567890', b'10:1234567890')
+        self._check(b"", b"0:")
+        self._check(b"abc", b"3:abc")
+        self._check(b"1234567890", b"10:1234567890")
 
     def test_large_string(self):
         self.assertRaises(ValueError, self.module.bdecode, b"2147483639:foo")
 
     def test_malformed_string(self):
-        self._run_check_error(ValueError, b'10:x')
-        self._run_check_error(ValueError, b'10:')
-        self._run_check_error(ValueError, b'10')
-        self._run_check_error(ValueError, b'01:x')
-        self._run_check_error(ValueError, b'00:')
-        self._run_check_error(ValueError, b'35208734823ljdahflajhdf')
-        self._run_check_error(ValueError, b'432432432432432:foo')
-        self._run_check_error(ValueError, b' 1:x')  # leading whitespace
-        self._run_check_error(ValueError, b'-1:x')  # negative
-        self._run_check_error(ValueError, b'1 x')  # space vs colon
-        self._run_check_error(ValueError, b'1x')  # missing colon
-        self._run_check_error(ValueError, (b'1' * 1000) + b':')
+        self._run_check_error(ValueError, b"10:x")
+        self._run_check_error(ValueError, b"10:")
+        self._run_check_error(ValueError, b"10")
+        self._run_check_error(ValueError, b"01:x")
+        self._run_check_error(ValueError, b"00:")
+        self._run_check_error(ValueError, b"35208734823ljdahflajhdf")
+        self._run_check_error(ValueError, b"432432432432432:foo")
+        self._run_check_error(ValueError, b" 1:x")  # leading whitespace
+        self._run_check_error(ValueError, b"-1:x")  # negative
+        self._run_check_error(ValueError, b"1 x")  # space vs colon
+        self._run_check_error(ValueError, b"1x")  # missing colon
+        self._run_check_error(ValueError, (b"1" * 1000) + b":")
 
     def test_list(self):
-        self._check([], b'le')
-        self._check([b'', b'', b''], b'l0:0:0:e')
-        self._check([1, 2, 3], b'li1ei2ei3ee')
-        self._check([b'asd', b'xy'], b'l3:asd2:xye')
-        self._check([[b'Alice', b'Bob'], [2, 3]], b'll5:Alice3:Bobeli2ei3eee')
+        self._check([], b"le")
+        self._check([b"", b"", b""], b"l0:0:0:e")
+        self._check([1, 2, 3], b"li1ei2ei3ee")
+        self._check([b"asd", b"xy"], b"l3:asd2:xye")
+        self._check([[b"Alice", b"Bob"], [2, 3]], b"ll5:Alice3:Bobeli2ei3eee")
 
     def test_list_deepnested(self):
         import platform
-        if (platform.python_implementation() == 'PyPy'
-                or sys.version_info[:2] >= (3, 12)):
+
+        if platform.python_implementation() == "PyPy" or sys.version_info[
+            :2
+        ] >= (3, 12):
             expected = []
             for i in range(99):
                 expected = [expected]
@@ -296,49 +301,56 @@ class TestBencodeDecode(TestCase):
         else:
             with RecursionLimit():
                 self._run_check_error(
-                    RuntimeError, (b"l" * 100) + (b"e" * 100))
+                    RuntimeError, (b"l" * 100) + (b"e" * 100)
+                )
 
     def test_malformed_list(self):
-        self._run_check_error(ValueError, b'l')
-        self._run_check_error(ValueError, b'l01:ae')
-        self._run_check_error(ValueError, b'l0:')
-        self._run_check_error(ValueError, b'li1e')
-        self._run_check_error(ValueError, b'l-3:e')
+        self._run_check_error(ValueError, b"l")
+        self._run_check_error(ValueError, b"l01:ae")
+        self._run_check_error(ValueError, b"l0:")
+        self._run_check_error(ValueError, b"li1e")
+        self._run_check_error(ValueError, b"l-3:e")
 
     def test_dict(self):
-        self._check({}, b'de')
-        self._check({b'': 3}, b'd0:i3ee')
-        self._check({b'age': 25, b'eyes': b'blue'}, b'd3:agei25e4:eyes4:bluee')
-        self._check({b'spam.mp3': {b'author': b'Alice', b'length': 100000}},
-                    b'd8:spam.mp3d6:author5:Alice6:lengthi100000eee')
+        self._check({}, b"de")
+        self._check({b"": 3}, b"d0:i3ee")
+        self._check({b"age": 25, b"eyes": b"blue"}, b"d3:agei25e4:eyes4:bluee")
+        self._check(
+            {b"spam.mp3": {b"author": b"Alice", b"length": 100000}},
+            b"d8:spam.mp3d6:author5:Alice6:lengthi100000eee",
+        )
 
     def test_dict_deepnested(self):
+        if self.id().endswith("(C)") and sys.version_info >= (3, 12):
+            self.skipTest("Python 3.12+ does not limit recursion in C code")
+
         with RecursionLimit():
             self._run_check_error(
-                RuntimeError, (b"d0:" * 1000) + b'i1e' + (b"e" * 1000))
+                RuntimeError, (b"d0:" * 1000) + b"i1e" + (b"e" * 1000)
+            )
 
     def test_malformed_dict(self):
-        self._run_check_error(ValueError, b'd')
-        self._run_check_error(ValueError, b'defoobar')
-        self._run_check_error(ValueError, b'd3:fooe')
-        self._run_check_error(ValueError, b'di1e0:e')
-        self._run_check_error(ValueError, b'd1:b0:1:a0:e')
-        self._run_check_error(ValueError, b'd1:a0:1:a0:e')
-        self._run_check_error(ValueError, b'd0:0:')
-        self._run_check_error(ValueError, b'd0:')
-        self._run_check_error(ValueError, b'd432432432432432432:e')
+        self._run_check_error(ValueError, b"d")
+        self._run_check_error(ValueError, b"defoobar")
+        self._run_check_error(ValueError, b"d3:fooe")
+        self._run_check_error(ValueError, b"di1e0:e")
+        self._run_check_error(ValueError, b"d1:b0:1:a0:e")
+        self._run_check_error(ValueError, b"d1:a0:1:a0:e")
+        self._run_check_error(ValueError, b"d0:0:")
+        self._run_check_error(ValueError, b"d0:")
+        self._run_check_error(ValueError, b"d432432432432432432:e")
 
     def test_empty_string(self):
-        self.assertRaises(ValueError, self.module.bdecode, b'')
+        self.assertRaises(ValueError, self.module.bdecode, b"")
 
     def test_junk(self):
-        self._run_check_error(ValueError, b'i6easd')
-        self._run_check_error(ValueError, b'2:abfdjslhfld')
-        self._run_check_error(ValueError, b'0:0:')
-        self._run_check_error(ValueError, b'leanfdldjfh')
+        self._run_check_error(ValueError, b"i6easd")
+        self._run_check_error(ValueError, b"2:abfdjslhfld")
+        self._run_check_error(ValueError, b"0:0:")
+        self._run_check_error(ValueError, b"leanfdldjfh")
 
     def test_unknown_object(self):
-        self.assertRaises(ValueError, self.module.bdecode, b'relwjhrlewjh')
+        self.assertRaises(ValueError, self.module.bdecode, b"relwjhrlewjh")
 
     def test_unsupported_type(self):
         self._run_check_error(TypeError, 1.5)
@@ -352,7 +364,6 @@ class TestBencodeDecode(TestCase):
 
 
 class TestBdecodeUtf8(TestCase):
-
     module = None
 
     def _check(self, expected, source):
@@ -363,67 +374,70 @@ class TestBdecodeUtf8(TestCase):
         self.assertRaises(exc, self.module.bdecode_utf8, bad)
 
     def test_string(self):
-        self._check('', b'0:')
-        self._check('aäc', b'4:a\xc3\xa4c')
-        self._check('1234567890', b'10:1234567890')
+        self._check("", b"0:")
+        self._check("aäc", b"4:a\xc3\xa4c")
+        self._check("1234567890", b"10:1234567890")
 
     def test_large_string(self):
         self.assertRaises(
-            ValueError, self.module.bdecode_utf8, b"2147483639:foo")
+            ValueError, self.module.bdecode_utf8, b"2147483639:foo"
+        )
 
     def test_malformed_string(self):
-        self._run_check_error(ValueError, b'10:x')
-        self._run_check_error(ValueError, b'10:')
-        self._run_check_error(ValueError, b'10')
-        self._run_check_error(ValueError, b'01:x')
-        self._run_check_error(ValueError, b'00:')
-        self._run_check_error(ValueError, b'35208734823ljdahflajhdf')
-        self._run_check_error(ValueError, b'432432432432432:foo')
-        self._run_check_error(ValueError, b' 1:x')  # leading whitespace
-        self._run_check_error(ValueError, b'-1:x')  # negative
-        self._run_check_error(ValueError, b'1 x')  # space vs colon
-        self._run_check_error(ValueError, b'1x')  # missing colon
-        self._run_check_error(ValueError, (b'1' * 1000) + b':')
+        self._run_check_error(ValueError, b"10:x")
+        self._run_check_error(ValueError, b"10:")
+        self._run_check_error(ValueError, b"10")
+        self._run_check_error(ValueError, b"01:x")
+        self._run_check_error(ValueError, b"00:")
+        self._run_check_error(ValueError, b"35208734823ljdahflajhdf")
+        self._run_check_error(ValueError, b"432432432432432:foo")
+        self._run_check_error(ValueError, b" 1:x")  # leading whitespace
+        self._run_check_error(ValueError, b"-1:x")  # negative
+        self._run_check_error(ValueError, b"1 x")  # space vs colon
+        self._run_check_error(ValueError, b"1x")  # missing colon
+        self._run_check_error(ValueError, (b"1" * 1000) + b":")
 
     def test_empty_string(self):
-        self.assertRaises(ValueError, self.module.bdecode_utf8, b'')
+        self.assertRaises(ValueError, self.module.bdecode_utf8, b"")
 
     def test_invalid_utf8(self):
-        self._run_check_error(UnicodeDecodeError, b'3:\xff\xfe\xfd')
+        self._run_check_error(UnicodeDecodeError, b"3:\xff\xfe\xfd")
 
 
 class TestBencodeEncode(TestCase):
-
     module = None
 
     def _check(self, expected, source):
         self.assertEqual(expected, self.module.bencode(source))
 
     def test_int(self):
-        self._check(b'i4e', 4)
-        self._check(b'i0e', 0)
-        self._check(b'i-10e', -10)
+        self._check(b"i4e", 4)
+        self._check(b"i0e", 0)
+        self._check(b"i-10e", -10)
 
     def test_long(self):
-        self._check(b'i12345678901234567890e', 12345678901234567890)
-        self._check(b'i-12345678901234567890e', -12345678901234567890)
+        self._check(b"i12345678901234567890e", 12345678901234567890)
+        self._check(b"i-12345678901234567890e", -12345678901234567890)
 
     def test_string(self):
-        self._check(b'0:', b'')
-        self._check(b'3:abc', b'abc')
-        self._check(b'10:1234567890', b'1234567890')
+        self._check(b"0:", b"")
+        self._check(b"3:abc", b"abc")
+        self._check(b"10:1234567890", b"1234567890")
 
     def test_list(self):
-        self._check(b'le', [])
-        self._check(b'li1ei2ei3ee', [1, 2, 3])
-        self._check(b'll5:Alice3:Bobeli2ei3eee', [[b'Alice', b'Bob'], [2, 3]])
+        self._check(b"le", [])
+        self._check(b"li1ei2ei3ee", [1, 2, 3])
+        self._check(b"ll5:Alice3:Bobeli2ei3eee", [[b"Alice", b"Bob"], [2, 3]])
 
     def test_list_as_tuple(self):
-        self._check(b'le', ())
-        self._check(b'li1ei2ei3ee', (1, 2, 3))
-        self._check(b'll5:Alice3:Bobeli2ei3eee', ((b'Alice', b'Bob'), (2, 3)))
+        self._check(b"le", ())
+        self._check(b"li1ei2ei3ee", (1, 2, 3))
+        self._check(b"ll5:Alice3:Bobeli2ei3eee", ((b"Alice", b"Bob"), (2, 3)))
 
     def test_list_deep_nested(self):
+        if self.id().endswith("(C)") and sys.version_info >= (3, 12):
+            self.skipTest("Python 3.12+ does not limit recursion in C code")
+
         top = []
         lst = top
         for unused_i in range(1000):
@@ -433,54 +447,60 @@ class TestBencodeEncode(TestCase):
             self.assertRaises(RuntimeError, self.module.bencode, top)
 
     def test_dict(self):
-        self._check(b'de', {})
-        self._check(b'd3:agei25e4:eyes4:bluee', {b'age': 25, b'eyes': b'blue'})
-        self._check(b'd8:spam.mp3d6:author5:Alice6:lengthi100000eee',
-                    {b'spam.mp3': {b'author': b'Alice', b'length': 100000}})
+        self._check(b"de", {})
+        self._check(b"d3:agei25e4:eyes4:bluee", {b"age": 25, b"eyes": b"blue"})
+        self._check(
+            b"d8:spam.mp3d6:author5:Alice6:lengthi100000eee",
+            {b"spam.mp3": {b"author": b"Alice", b"length": 100000}},
+        )
 
     def test_dict_deep_nested(self):
+        if self.id().endswith("(C)") and sys.version_info >= (3, 12):
+            self.skipTest("Python 3.12+ does not limit recursion in C code")
+
         d = top = {}
         for i in range(1000):
-            d[b''] = {}
-            d = d[b'']
+            d[b""] = {}
+            d = d[b""]
         with RecursionLimit():
             self.assertRaises(RuntimeError, self.module.bencode, top)
 
     def test_bencached(self):
-        self._check(b'i3e', self.module.Bencached(self.module.bencode(3)))
+        self._check(b"i3e", self.module.Bencached(self.module.bencode(3)))
 
     def test_invalid_dict(self):
         self.assertRaises(TypeError, self.module.bencode, {1: b"foo"})
 
     def test_bool(self):
-        self._check(b'i1e', True)
-        self._check(b'i0e', False)
+        self._check(b"i1e", True)
+        self._check(b"i0e", False)
 
 
 class TestBencodeEncodeUtf8(TestCase):
-
     module = None
 
     def _check(self, expected, source):
         self.assertEqual(expected, self.module.bencode_utf8(source))
 
     def test_string(self):
-        self._check(b'0:', '')
-        self._check(b'3:abc', 'abc')
-        self._check(b'10:1234567890', '1234567890')
+        self._check(b"0:", "")
+        self._check(b"3:abc", "abc")
+        self._check(b"10:1234567890", "1234567890")
 
     def test_list(self):
-        self._check(b'le', [])
-        self._check(b'li1ei2ei3ee', [1, 2, 3])
-        self._check(b'll5:Alice3:Bobeli2ei3eee', [['Alice', 'Bob'], [2, 3]])
+        self._check(b"le", [])
+        self._check(b"li1ei2ei3ee", [1, 2, 3])
+        self._check(b"ll5:Alice3:Bobeli2ei3eee", [["Alice", "Bob"], [2, 3]])
 
     def test_list_as_tuple(self):
-        self._check(b'le', ())
-        self._check(b'li1ei2ei3ee', (1, 2, 3))
-        self._check(b'll5:Alice3:Bobeli2ei3eee', (('Alice', 'Bob'), (2, 3)))
+        self._check(b"le", ())
+        self._check(b"li1ei2ei3ee", (1, 2, 3))
+        self._check(b"ll5:Alice3:Bobeli2ei3eee", (("Alice", "Bob"), (2, 3)))
 
     def test_dict(self):
-        self._check(b'de', {})
-        self._check(b'd3:agei25e4:eyes4:bluee', {b'age': 25, b'eyes': 'blue'})
-        self._check(b'd8:spam.mp3d6:author5:Alice6:lengthi100000eee',
-                    {b'spam.mp3': {b'author': b'Alice', b'length': 100000}})
+        self._check(b"de", {})
+        self._check(b"d3:agei25e4:eyes4:bluee", {b"age": 25, b"eyes": "blue"})
+        self._check(
+            b"d8:spam.mp3d6:author5:Alice6:lengthi100000eee",
+            {b"spam.mp3": {b"author": b"Alice", b"length": 100000}},
+        )
